@@ -351,7 +351,10 @@ class GnomeCanvasItem(_gtk.GtkObject):
 		_gnomeui.gnome_canvas_item_set(self._o, args)
 	def move(self, dx, dy):
 		_gnomeui.gnome_canvas_item_move(self._o, dx, dy)
-	# scale, rotate and transform in here
+	def affine_relative(self, affine):
+		_gnomeui.gnome_canvas_item_affine_relative(self._o, affine)
+	def affine_absolute(self, affine):
+		_gnomeui.gnome_canvas_item_affine_absolute(self._o, affine)
 	def raise_(self, positions):
 		_gnomeui.gnome_canvas_item_raise(self._o, positions)
 	def lower(self, positions):
