@@ -1,3 +1,22 @@
+/* gnome-python - python bindings for gnome-libs
+ * Copyright (C) 1998-1999 James Henstridge <james@daa.com.au>
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
+ */
+
 #include <Python.h>
 #include "libgnome/libgnome.h"
 #include "libgnome/gnome-fileconvert.h"
@@ -345,6 +364,20 @@ static PyObject *_wrap_gnome_metadata_type_remove(PyObject *self, PyObject *args
   Py_INCREF(Py_None);
   return Py_None;
 }
+static PyObject *_wrap_gnome_metadata_lock(PyObject *self, PyObject *args) {
+  if (!PyArg_ParseTuple(args, ":gnome_metadata_lock"))
+    return NULL;
+  gnome_metadata_lock();
+  Py_INCREF(Py_None);
+  return Py_None;
+}
+static PyObject *_wrap_gnome_metadata_unlock(PyObject *self, PyObject *args) {
+  if (!PyArg_ParseTuple(args, ":gnome_metadata_unlock"))
+    return NULL;
+  gnome_metadata_unlock();
+  Py_INCREF(Py_None);
+  return Py_None;
+}
 
 static PyObject *_wrap_gnome_mime_get_keys(PyObject *self, PyObject *args) {
   char *mime_type;
@@ -495,6 +528,8 @@ static PyMethodDef _gnomeMethods[] = {
     { "gnome_metadata_regex_remove", _wrap_gnome_metadata_regex_remove, 1 },
     { "gnome_metadata_type_add", _wrap_gnome_metadata_type_add, 1 },
     { "gnome_metadata_type_remove", _wrap_gnome_metadata_type_remove, 1 },
+    { "gnome_metadata_lock", _wrap_gnome_metadata_lock, 1 },
+    { "gnome_metadata_unlock", _wrap_gnome_metadata_unlock, 1 },
     { "gnome_mime_get_keys", _wrap_gnome_mime_get_keys, 1 },
     { "gnome_score_log", _wrap_gnome_score_log, 1 },
     { "gnome_score_get_notable", _wrap_gnome_score_get_notable, 1 },
