@@ -22,6 +22,8 @@
 
 #include <libgnomeui/libgnomeui.h>
 
+static gboolean PyGtk_FatalExceptions = FALSE;
+
 /* -- from other modules -- */
 typedef struct {
   PyObject_HEAD
@@ -567,7 +569,7 @@ static PyObject *_wrap_gnome_canvas_item_set(PyObject *self, PyObject *args) {
   GtkType type;
   gint nargs;
   GtkArg *arg;
-  if (!PyArg_ParseTuple(args, "O!O!:gnome_canvas_item_new", PyGtk_Type,
+  if (!PyArg_ParseTuple(args, "O!O!:gnome_canvas_item_set", PyGtk_Type,
 			&obj, &PyDict_Type, &dict))
     return NULL;
   type = GTK_OBJECT_TYPE(PyGtk_Get(obj));
@@ -580,6 +582,60 @@ static PyObject *_wrap_gnome_canvas_item_set(PyObject *self, PyObject *args) {
   return Py_None;
 }
 
+static PyObject *_wrap_gnome_canvas_item_affine_relative(PyObject *self, PyObject *args) {
+  PyObject *item, *py_affine;
+  double affine[6];
+  int i;
+
+  if (!PyArg_ParseTuple(args, "O!O:gnome_canvas_item_affine_relative",
+			PyGtk_Type, &item, &py_affine))
+    return NULL;
+  if (!PySequence_Check(py_affine) || PySequence_Length(py_affine) != 6) {
+    PyErr_SetString(PyExc_TypeError, "second argument must be a sequence, length 6");
+    return NULL;
+  }
+  for (i = 0; i < 6; i++) {
+    PyObject *sitem = PyNumber_Float(PySequence_GetItem(py_affine, i));
+    if (sitem)
+      affine[i] = PyFloat_AsDouble(sitem);
+    else {
+      PyErr_Clear();
+      PyErr_SetString(PyExc_TypeError, "sequence item not a float");
+      return NULL;
+    }
+  }
+  gnome_canvas_item_affine_relative(GNOME_CANVAS_ITEM(PyGtk_Get(item)),affine);
+  Py_INCREF(Py_None);
+  return Py_None;
+}
+
+static PyObject *_wrap_gnome_canvas_item_affine_absolute(PyObject *self, PyObject *args) {
+  PyObject *item, *py_affine;
+  double affine[6];
+  int i;
+
+  if (!PyArg_ParseTuple(args, "O!O:gnome_canvas_item_affine_absolute",
+			PyGtk_Type, &item, &py_affine))
+    return NULL;
+  if (!PySequence_Check(py_affine) || PySequence_Length(py_affine) != 6) {
+    PyErr_SetString(PyExc_TypeError, "second argument must be a sequence, length 6");
+    return NULL;
+  }
+  for (i = 0; i < 6; i++) {
+    PyObject *sitem = PyNumber_Float(PySequence_GetItem(py_affine, i));
+    if (sitem)
+      affine[i] = PyFloat_AsDouble(sitem);
+    else {
+      PyErr_Clear();
+      PyErr_SetString(PyExc_TypeError, "sequence item not a float");
+      return NULL;
+    }
+  }
+  gnome_canvas_item_affine_absolute(GNOME_CANVAS_ITEM(PyGtk_Get(item)),affine);
+  Py_INCREF(Py_None);
+  return Py_None;
+}
+
 static int read_vector(PyObject *obj, int *argc, char ***argv) {
   int i;
   if (obj == Py_None) {
@@ -693,8 +749,12 @@ static void PyGnome_interact_function(gpointer a, PyObject *func, int nargs,
 			      *GTK_RETLOC_INT(args[2]),
 			      *GTK_RETLOC_ENUM(args[3]));
   if (ret == NULL) {
-    PyErr_Print();
-    PyErr_Clear();
+    if (PyGtk_FatalExceptions)
+        gtk_main_quit();
+    else {
+        PyErr_Print();
+        PyErr_Clear();
+    }
     return;
   }
   Py_DECREF(ret);
@@ -1092,8 +1152,12 @@ static GtkWidget *PyGtk_cauldron_callback(GtkWidget *self, PyObject *tuple) {
   ret = PyObject_CallObject(PyTuple_GetItem(tuple, 0), params);
   Py_DECREF(params);
   if (ret == NULL) {
-    PyErr_Print();
-    PyErr_Clear();
+    if (PyGtk_FatalExceptions)
+        gtk_main_quit();
+    else {
+        PyErr_Print();
+        PyErr_Clear();
+    }
     return NULL;
   }
 
@@ -1257,8 +1321,12 @@ static void PyGnome_StringCallback(gchar *string, gpointer data) {
   if (ret)
     Py_DECREF(ret);
   else {
-    PyErr_Print();
-    PyErr_Clear();
+    if (PyGtk_FatalExceptions)
+      gtk_main_quit();
+    else {
+      PyErr_Print();
+      PyErr_Clear();
+    }
   }
   Py_DECREF(func); /* callback only called once */
 }
@@ -1270,8 +1338,12 @@ static void PyGnome_ReplyCallback(gint reply, gpointer data) {
   if (ret)
     Py_DECREF(ret);
   else {
-    PyErr_Print();
-    PyErr_Clear();
+    if (PyGtk_FatalExceptions)
+      gtk_main_quit();
+    else {
+      PyErr_Print();
+      PyErr_Clear();
+    }
   }
   Py_DECREF(func); /* callback only called once */
 }
@@ -1460,6 +1532,8 @@ static PyMethodDef gnomeuiMethods[] = {
     { "gnome_canvas_get_color", _wrap_gnome_canvas_get_color, 1 },
     { "gnome_canvas_item_new", _wrap_gnome_canvas_item_new, 1 },
     { "gnome_canvas_item_set", _wrap_gnome_canvas_item_set, 1 },
+    { "gnome_canvas_item_affine_relative", _wrap_gnome_canvas_item_affine_relative, 1 },
+    { "gnome_canvas_item_affine_absolute", _wrap_gnome_canvas_item_affine_absolute, 1 },
     { "gnome_client_set_clone_command", _wrap_gnome_client_set_clone_command, 1 },
     { "gnome_client_set_discard_command", _wrap_gnome_client_set_discard_command, 1 },
     { "gnome_client_set_restart_command", _wrap_gnome_client_set_restart_command, 1 },
@@ -1506,6 +1580,10 @@ void init_gnomeui() {
     m = Py_InitModule("_gnomeui", gnomeuiMethods);
 
     m = PyImport_ImportModule("_gtk");
+    if (m == NULL) {
+        Py_FatalError("couldn't import _gtk");
+        return;
+    }
     d = PyModule_GetDict(m);
     Py_DECREF(m);
 
@@ -1560,6 +1638,10 @@ void init_gnomeui() {
 						"PyGtk_RegisterBoxed"));
 
     m = PyImport_ImportModule("_gdkimlib");
+    if (m == NULL) {
+        Py_FatalError("couldn't import _gdkimlib");
+        return;
+    }
     d = PyModule_GetDict(m);
     Py_DECREF(m);
     PyGdkImlibImage_Type = (PyTypeObject *)PyDict_GetItemString(d,
@@ -1570,6 +1652,20 @@ void init_gnomeui() {
     PyGdkImlibImage_New = PyCObject_AsVoidPtr(PyDict_GetItemString(d,
 						"PyGdkImlibImage_New"));
 
+     m = PyImport_ImportModule("os");
+     if (m == NULL) {
+         Py_FatalError("couldn't import os");
+         return;
+     }
+     d = PyModule_GetDict(m);
+     Py_DECREF(m);
+     d = PyDict_GetItemString(d, "environ");
+     d = PyMapping_GetItemString(d, "PYGTK_FATAL_EXCEPTIONS");
+     if (d == NULL)
+         PyErr_Clear();
+     else
+         PyGtk_FatalExceptions = PyObject_IsTrue(d);
+
     if (PyErr_Occurred())
         Py_FatalError("can't initialise module _gnomeui");
 }
