@@ -183,6 +183,10 @@ void init_gtkxmhtml() {
     m = Py_InitModule("_gtkxmhtml", gtkxmhtmlMethods);
 
     m = PyImport_ImportModule("_gtk");
+    if (m == NULL) {
+        Py_FatalError("couldn't import _gtk");
+        return;
+    }
     d = PyModule_GetDict(m);
     Py_DECREF(m);
 
